<?php

namespace itguild\forms\app\controllers;

use Exception;
use itguild\forms\app\core\BaseController;

use itguild\forms\app\models\FormInputModel;
use itguild\forms\app\models\FormModel;
use itguild\forms\app\models\FormResModel;
use itguild\forms\app\models\InputValueModel;
use itguild\forms\app\models\User;
use itguild\forms\debug\Debug;
use itguild\forms\exceptions\FormNotFoundException;
use itguild\forms\Form;
use itguild\forms\JsonForm;
use itguild\forms\widgets\PaginationWidget;
use itguild\forms\widgets\ResultWidget;
use itguild\forms\widgets\TableJsonWidget;
use Twig\TwigFunction;


class FormController extends BaseController
{
    public function indexAction($id): void
    {
        // Инициализируем пустые массивы
        $meta = [];
        $formArr = [];
        $formFieldsArr = [];
        // Получение формы из БД и обработка мета данных формы
        $form = FormModel::find($id);

        if (!$form) {
            throw new FormNotFoundException($id);
        }

        if (isset($form->params)) {
            $allParams = json_decode($form->params, true);
            $meta = array_merge($meta, $allParams['params']);

        }
        $meta['action'] = "/form-save/" . $form->id;
        $meta['method'] = "POST";
        $formArr['meta'] = $meta;

        // Обработка полей формы
        if ($form->fields) {
            $fields = $form->fields;
            foreach ($fields as $field) {
                $options = [];
                $fieldArr = [];
                $params = [];
                if ($field->inputValue) {
                    foreach ($field->inputValue as $value) {
                        $options[$value['id']] = $value['value'];
                    }
                }

                if ($field->params) {
                    $fieldArr = array_merge($fieldArr, json_decode($field->params, true));
                }
                $fieldArr['name'] = $field->name ?? $fieldArr['name'];
                $fieldArr['type'] = $field->inputType['type'] ? $field->inputType['type'] : "textInput";
                $fieldArr['options'] = $options;
                $formArr['data'][] = $fieldArr;
            }
        }

        $formArr['data'][] = ['type' => "button", 'typeInput' => "submit", "value" => "Отправить", "name" => "", "class" => "btn btn-primary"];

        //Debug::dd($fields);

        $this->view->addFunction(new TwigFunction("create_form", function () use ($formArr) {
            $form = new JsonForm(json_encode($formArr));
            $form->convertHTML();
            $form->render();
        }));

        echo $this->view->render('form/form.html.twig', ['title' => $form->title]);

    }

    public function saveAction($id): void
    {
        $form = FormResModel::Create(['form_id' => $id, 'data' => json_encode($_POST),]);
    }


    public function createFormAction()
    {
        $form = Form::Create(['title' => "dsds", 'status' => 1, 'params' => "",]);
    }

    public function result($id, $page = 1)
    {

        $meta = [];
        $data = [];
        $tableArr = [];
        $columns = [];
        $columnsToMod = [];

        $form = FormModel::find($id);
        $formParams = json_decode($form->params, true);
        $perPage = $formParams['meta']['perpage'] ?? "10";
        $fillable = $formParams["meta"]["fillable"] ?? null;
        $offset = ($page - 1) * $perPage;
        $countItems = FormResModel::count();

        $table = FormResModel::where("form_id", $id)->limit(3)->offset($offset)->get();
        foreach ($table as $key => $item) {
            $data[] = array_merge(json_decode($item->data, true), ["id" => $item->id]);

        }

        if ($form->fields) {
            foreach ($form->fields as $field) {
                if ($field->input_type_id === 6){
                    $columnsToMod[] = $field->name;

                }
                $columns[$field->name] = $field->label;


            }
        }

        $meta['title'] = "Form ID: " . $id;
        $meta['columns'] = $columns;
        $meta["perPage"] = $perPage;
        $meta["fillable"] = $fillable;
        $meta['params'] = ["class" => "table table-bordered", "border" => "1"];
        $meta['currentPage'] = $page;
        $meta['baseUrl'] = "http://forms.local/";

        $tableArr["meta"] = $meta;
        $tableArr["data"] = $data;
        //Debug::prn($tableArr);
        $tableRes = json_encode($tableArr);
        $this->view->addFunction(new TwigFunction("create_table", function () use ($tableRes, $columnsToMod, $form) {

            $table = new TableJsonWidget($tableRes);


            $table->setBeforePrintCell(function ($column, $data) use ($columnsToMod, $form){
                if (in_array($column, $columnsToMod)){
                    $res = InputValueModel::find($data);
                    return $res->value ?? "";
                }

                return $data;
            });
            $table->create();
            $table->render();



        }));

        $this->view->addFunction(new TwigFunction("create_pagination", function () use ($perPage, $page, $countItems, $form) {

            $pagination = new PaginationWidget($countItems, $perPage, $page, "http://forms.local/form-load/$form->id");
            $pagination->create();
            $pagination->render();

        }));
        echo $this->view->render('form/result.html.twig', ['title' => $tableArr["meta"]["title"]]);

    }

    public function getResultItem($id)
    {
        $resultForm = FormResModel::find($id);
        $formID = $resultForm->form_id;

        $form = FormModel::find($formID);
        $meta = [];
        $columnsToMod = [];
        $columns = [];
        $resultData = $resultForm->data;


        if ($form->fields) {
            foreach ($form->fields as $field) {
                if ($field->input_type_id === 6){
                    $columnsToMod[] = $field->name;

                }
                $columns[$field->name] = $field->label;


            }
        }


        $meta['title'] = "ITEM ID: " . $id;
        $meta['columns'] = $columns;
        $meta['params'] = ["class" => "table table-bordered", "border" => "1"];


        $tableArr["meta"] = $meta;
        $tableArr["data"] = $resultForm;

        $tableRes = json_encode($tableArr);




        $this->view->addFunction(new TwigFunction("create_get_action", function () use ($tableRes, $columnsToMod, $form) {

            $pagination = new ResultWidget($tableRes);
            $pagination->setBeforePrintCell(function ($column, $data) use ($columnsToMod, $form) {
                if (in_array($column, $columnsToMod)) {
                    $res = InputValueModel::find($data);
                    return $res->value ?? "";
                }

                return $data;
            });
            $pagination->create();
            $pagination->render();

        }));
        echo $this->view->render('form/resultItem.html.twig', ['title' => $tableArr["meta"]["title"]]);
    }

}