<?php

namespace itguild\forms\widgets;

use itguild\forms\app\models\FormModel;
use itguild\forms\app\models\FormResModel;
use itguild\forms\debug\Debug;
use itguild\forms\traits\CreateParams;

class TableJsonWidget extends BaseWidget
{
    use CreateParams;

    private string $html = '';
    private string $json;

    private int $count = 0;
    private \Closure|false $beforePrintCell;
    private string $baseUrl;
    private array $data;

    public function __construct(string $json)
    {
        $this->beforePrintCell = false;
        $this->json = $json;
        $this->data = json_decode($this->json, true);
        $this->baseUrl = $this->data['meta']['baseUrl'] ?? null;
    }

    public function beginTable(): void
    {
        $paramsStr = $this->createParams($this->data['meta']['params']);
        $this->html = "<table $paramsStr>";
    }

    public function createThead(): void
    {
        if (!isset($this->data['meta']['columns'])) {
            return;
        }

        $this->html .= "<thead><tr>";
        $this->html .= "<th>" . "ID" . "</th>";
        foreach ($this->data['meta']['columns'] as $key => $column) {
            if ($this->is_fillable($key)){
                $this->html .= "<th>" . $column . "</th>";
            }
        }
        $this->html .= "<th>Действия</th></th></tr></thead>";
    }

    public function createTbody(): void
    {
        if ($this->data['data']) {

            $this->count = $this->data["meta"]["perPage"] * ($this->data['meta']["currentPage"] - 1);
            foreach ($this->data['data'] as $col) {
                $this->html .= "<tr>";
                $this->count += 1;
                $this->html .= '<td><a href=' . $this->baseUrl . "form-item/" . $col["id"] . '>' . $this->count . '</a></td>';
                foreach ($col as $key => $row) {
                    if ($this->issetColumn($key) and $this->is_fillable($key)) {
                        if ($this->beforePrintCell) {
                            $hook = $this->beforePrintCell;
                            $row = $hook($key, $row);
                        }

                        $this->html .= "<td>" . $row . "</td>";
                    }
                }
                $this->html .= "<td><a href='" . $this->baseUrl . "form-item/" . $col["id"] . "' style='margin-right: 20px'>Просмотр</a><a href='#' style='margin-right: 20px'>Удалить</a></td></tr>";
            }
        }
    }

    private function issetColumn($column)
    {
        if (isset($this->data['meta']['columns'])) {
            foreach ($this->data['meta']['columns'] as $key => $currentColumn) {
                if ($key === $column) {
                    return true;
                }
            }
        }

        return false;
    }

    private function is_fillable($column)
    {
        if (in_array($column, $this->data['meta']['fillable'])) {
            return true;
        }

        return false;
    }

    public function create()
    {
        $this->beginTable();
        $this->createThead();
        $this->createTbody();
        $this->endTable();
    }

    public function tableAction($json): void
    {

        $tableJson = json_decode($json, true);

        foreach ($tableJson as $key => $value) {
            echo "<tr>";
            echo "<td>" . $key . "</td>";
            echo "<td>" . $value . "</td>";
            echo "</tr>";
        }
    }

    public function endTable(): void
    {
        $this->html .= "</table>";
    }

    public function render(): void
    {
        echo $this->html;
    }

    public function setBeforePrintCell(\Closure $closure): void
    {
        $this->beforePrintCell = $closure;
    }
}